"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Node_1 = require("./types/Node");
const combinator_1 = require("./combinator");
const parseBold = (0, combinator_1.explicit)((0, combinator_1.regexp)(/^\*(\S([^*\n]*?|[^*\n]*? `.*?` )[^\s*]|\S)\*(?=[\s~!#$%^)\-+={}[\];:'",.?/]|$)/, (match, text, position, parseText) => {
    const [matchedText, content] = match;
    return [
        {
            type: Node_1.NodeType.Bold,
            children: parseText(content),
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parseCode = (0, combinator_1.explicit)((0, combinator_1.regexp)(/^`([^`]+?)`(?=[\s.,\])}!?\-=]|$)/, (match, text, position) => {
    const [matchedText, content] = match;
    return [
        {
            type: Node_1.NodeType.Code,
            text: content,
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parsePreText = (0, combinator_1.explicit)((0, combinator_1.regexp)(/^```(\s*\S[\s\S]*?\s*)```(?=[\s.,\])}!?\-=]|$)/, (match, text, position) => {
    const [matchedText, content] = match;
    return [
        {
            type: Node_1.NodeType.PreText,
            text: content,
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parseItalic = (0, combinator_1.explicit)((0, combinator_1.regexp)(/^_(\S([^_\n]*?|[^_\n]*? `.*?` )\S|\S)\_(?=[\s.,\])}!?\-=]|$)/, (match, text, position, parseText) => {
    const [matchedText, content] = match;
    return [
        {
            type: Node_1.NodeType.Italic,
            children: parseText(content),
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parseStrike = (0, combinator_1.explicit)((0, combinator_1.regexp)(/^~(\S([^~\n]*?|[^~\n]*? `.*?` )\S|\S)\~(?=[\s.,\])}!?\-=]|$)/, (match, text, position, parseText) => {
    const [matchedText, content] = match;
    return [
        {
            type: Node_1.NodeType.Strike,
            children: parseText(content),
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parseSingleLineQuote = (0, combinator_1.topOfLine)((0, combinator_1.regexp)(/^&gt;(.*)(\n|$)/, (match, text, position, parseText) => {
    const [matchedText, content] = match;
    const repeatedGt = content.match(/^((&gt;)+)(.*)$/);
    return [
        {
            type: Node_1.NodeType.Quote,
            children: repeatedGt
                ? [
                    {
                        type: Node_1.NodeType.Text,
                        text: repeatedGt[1],
                        source: repeatedGt[1]
                    },
                    ...parseText(repeatedGt[3])
                ]
                : parseText(content),
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parseMultilineQuote = (0, combinator_1.topOfLine)((0, combinator_1.regexp)(/^&gt;&gt;&gt;([\s\S]+)$/, (match, text, position, parseText) => {
    const [matchedText, content] = match;
    return [
        {
            type: Node_1.NodeType.Quote,
            children: parseText(content),
            source: matchedText
        },
        position + matchedText.length
    ];
}));
const parseEmoji = (0, combinator_1.regexp)(/^:([^:<`*#@!\s()$%]+):(:(skin-tone-.+?):)?/, (match, text, position) => {
    const [matchedText, name, _, variation] = match;
    return [
        {
            type: Node_1.NodeType.Emoji,
            name,
            variation,
            source: matchedText
        },
        position + matchedText.length
    ];
});
const parseLink = (0, combinator_1.regexp)(/^<([^\s<>][^\n<>]*?)(\|([^<>]+?))?>/, (match, text, position, parseText) => {
    const [matchedText, link, _, label] = match;
    const nextPosition = position + matchedText.length;
    const labelNodes = label ? parseText(label) : undefined;
    switch (link.charAt(0)) {
        case '@':
            return [
                {
                    type: Node_1.NodeType.UserLink,
                    userID: link.slice(1),
                    label: labelNodes,
                    source: matchedText
                },
                nextPosition
            ];
        case '#':
            return [
                {
                    type: Node_1.NodeType.ChannelLink,
                    channelID: link.slice(1),
                    label: labelNodes,
                    source: matchedText
                },
                nextPosition
            ];
        case '!':
            const [commandName, ...args] = link.slice(1).split('^');
            return [
                {
                    type: Node_1.NodeType.Command,
                    name: commandName,
                    arguments: args,
                    label: labelNodes,
                    source: matchedText
                },
                nextPosition
            ];
        default:
            return [
                {
                    type: Node_1.NodeType.URL,
                    url: link,
                    label: labelNodes,
                    source: matchedText
                },
                nextPosition
            ];
    }
});
exports.default = (0, combinator_1.or)([
    parseBold,
    parsePreText,
    parseCode,
    parseEmoji,
    parseItalic,
    parseMultilineQuote,
    parseSingleLineQuote,
    parseLink,
    parseStrike
]);
//# sourceMappingURL=parser.js.map